// Making the required imports
import { csv, select,legendColor } from 'd3';
import { scatterPlot } from './scatterPlot';

//Loading the sales data
const csvUrl = [
  'https://gist.githubusercontent.com/',
  'maheganesh/', 
  'eaae5d72e5c5d223f77d9ac9fa53d1c6/', 
  'raw/d8a6c1dbda3da3246dc1149d63f63bea83a6f141/', 
  'PS4_GamesSales.csv', 
].join('');
const parseRow = (d) => {
  // Converting the quantivative data to numbers.
  d['Year'] = +d['Year'];
  d['Global'] = +d['Global'];
  d['North America'] = +d['North America'];
  return d;
};

const width = window.innerWidth;
const height = window.innerHeight;
// Creating an SVG for the graph
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// Defining the data used by the graphs
const xValue = (d) => d['Year'];
const yValue = (d) => d['Global'];
const rxValue = (d) => d['North America'];
const ryValue = (d) => d['Global'];
const cateogryValue = (d) => d.Category;
const colorValue = d => d.Category;
const colorLabel = 'Category';

// Main Function to create a reusable graph
const main = async () => {
  const data = await csv(csvUrl, parseRow);
  const x = xValue((d) => d['Year']);
  const y = yValue((d) => d['Global']);
  // Creating markers for the various categories of products.
  const marker = (d) => {
        	switch(d){
            case 'Garment': //label names on top
              return 'green';  // colours for each type
            case 'Electronics':
              return 'blue';
            case 'Decoration':
              return 'yellow';
            case 'Fitness':
              return 'pink';
            case 'Footwear':
              return 'red';  
            default:
              return 'orange';
          }
        }
  //values of xand y axis
  const marks = data.map((d) => ({
          x: xValue((d) => d['Year']),
          y: yValue((d) => d['Global']),
          species: cateogryValue(d),
          color: marker(cateogryValue(d)),
        	rx: rxValue(d),
          ry: ryValue(d),
                      
        }));
  
//structure of scatterplot  is defined
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d['Year'])
    .yValue((d) => d['Global'])
    .margin({
      top: 28,
      right: 53,
      bottom: 82,
      left: 88,

    })
    .radius(5);
  

// Adding color legends
 const g = svg.append('g')
          .attr('transform', `translate(${100},${100})`);
      const colorLegendG = g.append('g')
          .attr('transform', `translate(${100}, 150)`);

//shape of that scattered data 
svg
          .selectAll('circle')
          .data(marks)
          .join('circle')
          .attr('cx', (d) => d.x)
          .attr('cy', (d) => d.y)
        	.attr('fill', (d) => d.color)
        	.attr('rx', (d) => (d.rx))
        	.attr('ry', (d) => (d.ry));
  
// Columns selected for X Axis 
  const columns = [
    'Year',
    'North America',
  ];
// logic for changin the X Axis perodically
  let i = 0;
  setInterval(() => {
    plot.xValue((d) => d[columns[i % columns.length]]);  
		svg.selectAll('text').remove();
    svg.call(plot);
    
      colorLegendG.append('text')
          .attr('class', 'legend-label')
          .attr('x', -70)
          .attr('y', -210)
          .style("font-style","italic")
          .style("stroke","blue")
          .text(colorLabel);

   
// diaplay Title 
svg.append("text").attr("x", 450).attr("y", 20).text("Sales Scatterplot").style("font-size", "40px").attr("alignment-baseline","middle")

   
// Adding  labels in the legend and its style 
svg.append("text").attr("x", 150).attr("y", 70).text("Garment").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("circle").attr("cx",130).attr("cy",70).attr("r", 6).style("fill", "green")

svg.append("text").attr("x", 150).attr("y", 100).text("Electronics").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("circle").attr("cx",130).attr("cy",100).attr("r", 6).style("fill", "blue")

svg.append("text").attr("x", 150).attr("y", 130).text("Decoration").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("circle").attr("cx",130).attr("cy",130).attr("r", 6).style("fill", "yellow")

svg.append("text").attr("x", 150).attr("y", 160).text("Fitness").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("circle").attr("cx",130).attr("cy",160).attr("r", 6).style("fill", "pink")

svg.append("text").attr("x", 150).attr("y", 190).text("Footwear").style("font-size", "20px").attr("alignment-baseline","middle")
svg.append("circle").attr("cx",130).attr("cy",190).attr("r", 6).style("fill", "red")
  // Adding the Y Axis Lable
    svg
        	.append("text")
        	.attr('transform', 'rotate(-90)')
        	.attr('x',0 - (height /2))
        	.attr("y",  height/12)
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
        	.text("Sales");
    
    if(i % columns.length==0) 
    svg
        	.append("text")
        	.attr("x", width / 2 )
      		.attr("y",  480 )
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
      		.text("Year");
     // Number of Orders on x-axis
    else if(i % columns.length==1) 
    svg
        	.append("text")
        	.attr("x", width / 2 )
      		.attr("y",  480 )
        	.style("text-anchor", "middle")
        	.style("font-style","italic")
        	.style("font-weight","bold")
      		.text("North America");
     // Available in StockS on x-axis
    i++;
  }, 2000);// every 2 seconds change the visualization
};
main();